//+------------------------------------------------------------------+
//|                                                     MyTrader.mq4 |
//|                                         Copyright  2010, Geafer |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Geafer"
#property link      ""

// pipojen knihovny s funkcemi pro obchodovn
#include <TradeLib.mqh>

// poet lot, kter se maj obchodovat
extern double Lots = 0.1;
// poet pip pro nastaven SL
extern int StopLossPips = 30;
// poet pip, po kterch se m nastavovat SL
extern int TrailStopLossPips = 15;
// poet pip pro nastaven TP
extern int TakeProfitPips = 0;
// uruje, zda se m obchodovat nepetrit
extern bool Trade24H = true;
// hodina pro zahjen obchodu
extern int TradeHourStart = 6;
// minuta pro zahjen obchodu
extern int TradeMinuteStart = 30;
// hodina pro ukonen obchodu
extern int TradeHourStop = 22;
// minuta pro ukonen obchodu
extern int TradeMinuteStop = 30;

// mnovm pr, kter se obchoduje
string symbol;
// ID otevenho pokynu
int ticketId = -1;
// Uruje typ operace pokynu (True = BUY, jinak SELL)
bool isBuy;

// spodn hranice WPR
double wprLimitLow = -80;
// horn hranice WPR
double wprLimitHigh = -20;

// inicialilzace
int init()
{
   symbol = Symbol();
   return(0);
}

// deinicializace
int deinit()
{
   return(0);
}

// bh traderu
int start()
{
   // pokud nemme oteven, dn pokyn
   // zkontrolujeme, zda existuje njak signl
   if(ticketId == -1)
   {
      // zkontrolujeme signly pro oteven
      checkOpenSignals();
   }
   else
   {
      // pokud je pokyn stle oteven
      // zkontrolujeme signly pro zaven
      if(IsTicketOpen(ticketId))
      {
         checkCloseSignals();
         // pokud byl pokyn zaven
         // zkontrolujeme signly pro oteven
         if(ticketId == -1)
         {
            checkOpenSignals();
         }
         else
         {
            // jinak provedeme ppadn posunut SL
            TrailStopLoss(ticketId, TrailStopLossPips, StopLossPips, true);
         }
      }
      else
      {
         // pokyn ji nen oteven
         // zkontrolujeme signly pro oteven
         ticketId = -1;
         checkOpenSignals();
      }
   }
   
   return(0);
}

// ------------ pomocn funkce ---------------

// zjist, zda se me provdt obchodovn
bool canTrade()
{
   // kontrola, zda nebme v testovacm reimu a zda jsou zapl strategie
   if(IsTesting() == false)
   {
      if(IsExpertEnabled() == false)
      {
         Alert("Pokud chcete, aby EA bylo aktivn je teba zapnout strategie");
         return(false);
      }
   }

   if(Trade24H)
      return(true);
   else
      return(IsTimeToTrade(TradeHourStart, TradeMinuteStart, TradeHourStop, TradeMinuteStop));
}

// vrt hodnotu MA pro zelenou linii
double getGreenMA(int shift)
{
   return(iMA(symbol, Period(), 5, 0, MODE_EMA, PRICE_CLOSE, shift));
}

// vrt hodnotu MA pro ervenou linii
double getRedMA(int shift)
{
   return(iMA(symbol, Period(), 7, 0, MODE_EMA, PRICE_OPEN, shift));
}

// vrt hodnotu William's %R
double getWPR(int shift)
{
   return(iWPR(symbol, Period(), 40, shift));
}

// pomocn funkce, kter zjist, zda dochz ke ken linie A smrem nahoru pes linii B
// hodnoty jsou dny v parametrech
// hodnoty mus bt pro stejn asov psmo (A1 = B1 v ase, A2 = B2 v ase)
bool isCrossingUp(double valueA1, double valueA2, double valueB1, double valueB2)
{
   if(valueA1 <= valueA2 || valueA1 < valueB1 || valueA2 > valueB2)
      return(false);
   else
      return(true);
}

// pomocn funkce, kter zjist, zda dochz ke ken linie A smrem dolu pes linii B
// hodnoty jsou dny v parametrech
// hodnoty mus bt pro stejn asov psmo (A1 = B1 v ase, A2 = B2 v ase)
bool isCrossingDown(double valueA1, double valueA2, double valueB1, double valueB2)
{
   if(valueA1 >= valueA2 || valueA1 > valueB1 || valueA2 < valueB2)
      return(false);
   else
      return(true);
}

// provede kontrolu otevracch signl
void checkOpenSignals()
{
   if(canTrade() == false)
      return;
      
   if(isOpenBuySignal())          // kontrola na BUY signal
      isBuy = true;
   else if(isOpenSellSignal())    // kontrola na SELL signal
      isBuy = false;
   else
      return;
      
   ticketId = OpenTicket(symbol, isBuy, Lots, StopLossPips, TakeProfitPips);
}

// provede kontrolu zavracch signl
void checkCloseSignals()
{
   if(canTrade() == false)
      return;
      
   bool doClose = false;
   
   if(isBuy)
      doClose = isCloseBuySignal();
   else
      doClose = isCloseSellSignal();
   
   // pokud je as zavt pokyn, uzaveme jej
   if(doClose)
   {
      CloseTicket(ticketId);
      ticketId = -1;
   }
}

// zjist, zda je signl pro oteven BUY pozice
bool isOpenBuySignal()
{
   int shift = 1;                         // posuneme se o jednu svku zpt
   double maG1 = getGreenMA(shift);       // hodnota zelen MA
   double maG2 = getGreenMA(shift + 1);   // hodnota zelen MA o jednu star
   double maR1 = getRedMA(shift);         // hodnota erven MA
   double maR2 = getRedMA(shift + 1);     // hodnota erven MA o jednu star
   double wpr1 = getWPR(shift);           // hodnota WPR
   double wpr2 = getWPR(shift + 1);       // hodnota WPR o jednu star
   double wpr3 = getWPR(shift + 2);       // hodnota WPR o dv star
   
   // zjistme, zda je ken MA smrem nahoru
   // a zda WPR smuje pes spodn hranici smrem nahoru
   if(isCrossingUp(maG1, maG2, maR1, maR2) && 
      (wpr1 > wprLimitLow && (wpr2 < wprLimitLow || wpr3 < wprLimitLow)))
   {
      return(true);
   }
   else
   {
      return(false);
   }
}

// zjist, zda je signl pro zaven BUY pozice
bool isCloseBuySignal()
{
   return(isOpenSellSignal());
}

// zjist, zda je signl pro oteven SELL pozice
bool isOpenSellSignal()
{
   int shift = 1;                         // posuneme se o jednu svku zpt
   double maG1 = getGreenMA(shift);       // hodnota zelen MA
   double maG2 = getGreenMA(shift + 1);   // hodnota zelen MA o jednu star
   double maR1 = getRedMA(shift);         // hodnota erven MA
   double maR2 = getRedMA(shift + 1);     // hodnota erven MA o jednu star
   double wpr1 = getWPR(shift);           // hodnota WPR
   double wpr2 = getWPR(shift + 1);       // hodnota WPR o jednu star
   double wpr3 = getWPR(shift + 2);
   
   // zjistme, zda je ken MA smrem dolu
   // a zda WPR smuje pes horn hranici smrem dolu
   if(isCrossingDown(maG1, maG2, maR1, maR2) && 
      (wpr1 < wprLimitHigh && (wpr2 > wprLimitHigh || wpr3 > wprLimitHigh)))
   {
      return(true);
   }
   else
   {
      return(false);
   }
}

// zjist, zda je signl pro zaven SELL pozice
bool isCloseSellSignal()
{
   return(isOpenBuySignal());
}